rm(list = ls())
 
german_creditv <- read.csv(".../german_creditv1.0.csv", sep = "") # !!!insert working directory here !!!
Duration<-german_credit[,1] #response
Amount<-german_credit[,2] #covariate 1
Censoring<- german_credit[,3] # 0 =  good credit = censored because we do not know if they are going to default
age<-german_credit[,4] # covariate 2

SavingsAccount<-german_credit[,5]	# extra covariates for further experimentation
EmploymentHistory	<-german_credit[,6]
InstallmentRate	<-german_credit[,7]
NoOfCreditsInBank	<-german_credit[,8]
NoOfPeopleLiable<-german_credit[,9]

yyy<-seq(min(Duration), max(Duration), length=50) #grid points for covariates used for plotting / conditioning
xxx<-seq(min(Amount), max(Amount), length=50)
xxx2<-seq(min(age), max(age), length=50)
xxx3<-seq(min(SavingsAccount), max(SavingsAccount), length=50)
xxx4<-seq(min(InstallmentRate), max(InstallmentRate), length=50)
xxx5<-seq(min(EmploymentHistory), max(EmploymentHistory), length=50)


##### Local linear estimator functions, used for its implementation #######
#kernel functions:
"IntEpanechnikov"<-function(x)
{
  ifelse(x< -2.236068, 0, ifelse(x> 2.236068, 1, .5- (2.236068*x*(x^2-15))/100))
}

"IntUnif"<-function(x)
{
  ifelse(abs(x)< 1, 0.5*(x+1), ifelse(x> 1, 1, 0))
}

"Hstar"<-function(xin, cens, xout)
  # custom Kaplan-Meier function as defined in Marron and Padgett (1987), Annals of Statistics
{
  ordxin <- sort(xin) #order input data=Xi
  ordcens <- cens[order(xin)] #order censoring =Di
  n <- length(xin)
  nn <- length(xout)
  vKuse <- sapply(1:nn, function (i, ordxin, xout) max(c(max(which(ordxin <= xout[i]))
                                                         , min(which(xout[i] <= ordxin)))), ordxin, xout)
  vK <- vKuse - 1
  Huse<-sapply(vK, function (i,n,ordcens) ((n-1:i+1)/(n-1:i+2))^(1-ordcens[1:i]), n, ordcens)
  result<-sapply(Huse, prod)
  result 
}

"IntfHStar1"<-function(xin, cens, xout, h, kfun)
  # Empirical survival function estimator \Delta_i/(1-\hat H(X_i)) * K((y-Y_i)/h)
  # Experimental, used for testing purposes
{
  n<-length(xout)
  nn<-length(xin)
  out<-vector("numeric", n)
  denominator<- Hstar(xin, 1-cens, xin)
  arg1<-cens/ denominator 
  out<-sapply(1:n, function(i, xout, xin, h,arg1, kfun) 
    sum(kfun( (xout[i]-xin)/h)*arg1), xout, xin, h, arg1, kfun)
  out /(nn *h)
}

"csfe"<-function(yin, xin, yout, xout, h, b, kfun, wfun, cens)
  #Smooth survival function - \tilde F_L(y|x) (eq. 8 in article) for one covariate
{
  n<- length(xin)
  arg1<-(sapply(xout, "-", xin))/h  
  arg2<-kfun(arg1)
  
  arg3<-(sapply(yout, "-", yin))/b  
  arg3.1 <- cens/Hstar(yin, 1-cens, yin) 
  arg4<-wfun(arg3)* arg3.1
  
  #cat(nrow(arg2), " ", ncol(arg2), " ", nrow(arg4), " ", ncol(arg4), "\n")
  arg2<-matrix(arg2, nrow=nrow(arg4), ncol=ncol(arg4))
  arg5<- arg2 *arg4
  colSums(arg5)/n 
}

"csfebiv"<-function(yin, xin, yout, xout, h, b, kfun, wfun, cens)
  #Smooth survival function - \tilde F_L(y|x) (eq. 8 in article) for two covariates
{
  n<- nrow(xin)
  xin1<-xin[,1]
  xin2<-xin[,2]
  xout1<-xout[,1]
  xout2<-xout[,2]
  h1<-h[1]
  h2<-h[2]
  
  arg1.1<-  (sapply(xout1, "-", xin1))/h1
  arg2.1<-kfun(arg1.1)
  
  arg1.2<-      (sapply(xout2, "-", xin2))/h2
  arg2.2<-kfun(arg1.2)
  
  arg3<-   (sapply(yout, "-", yin))/b
  arg3.1 <- cens/Hstar(yin, 1-cens, yin)
  arg4<-wfun(arg3)* arg3.1
  arg2.1<-matrix(arg2.1, nrow=nrow(arg4), ncol=ncol(arg4))
  arg2.2<-matrix(arg2.2, nrow=nrow(arg4), ncol=ncol(arg4))
  arg5<- arg2.1 *arg2.2   *arg4
  colSums(arg5)/n
}

"kde"<-function(xin, xout, h)
  #classical kernel density estimator using the gaussian kernel
{
  n<- length(xin)
  arg1<-(sapply(xout, "-", xin))/h  
  arg2<-dnorm(arg1)
  arg3<-colSums(arg2)
  arg3/(n*h)
} 
 
 
########################
###MODELS S1, B1 version 1 - Fig. 1(B1)
par(mar=c(1 ,1,1,1))
mat.out<-matrix(nrow = length(xxx) , ncol = length(xxx) )
# MODEL S1:
for(i in 1: length(xxx))
{
  mat.out[,i]<- 1- csfe(Duration, Amount, yyy, xxx[i],  bw.SJ(Amount), 3,  pnorm, IntEpanechnikov, Censoring) 
}

persp(x=xxx, y=yyy, z=mat.out, theta = 70, phi = 25, ylab="Duration", xlab="Amount", zlab="Probability of Default", ticktype = "detailed")

mat.out<-matrix(nrow = length(xxx) , ncol = length(xxx) )
# MODEL S1:
for(i in 1: length(xxx2))
{
  aa<-1- csfe(Duration, age, yyy, xxx2[i],  bw.SJ(age), 3,  pnorm, IntEpanechnikov, Censoring)
  mat.out[,i]<- 1- csfe(Duration, age, yyy, xxx2[i],  bw.SJ(age), 3,  pnorm, IntEpanechnikov, Censoring) 
}
  
persp(x=xxx2, y=yyy, z=mat.out, theta = 70, phi = 25, ylab="Duration", xlab="Age", zlab="Probability of Default", ticktype = "detailed"  )


########################
###    Fig. 1 Graphs
########################
#B1 - fix amount
xbiv<-cbind(Amount, age)
for(i in 1: length(xxx2))
{
  a<-1- csfebiv(Duration, xbiv, yyy, cbind(xxx[10], xxx2[i]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)
  mat.out[,i]<-a
}
persp(x=xxx2, y=yyy, z=mat.out, theta = 70, phi = 20, ylab="Duration", xlab="Age", zlab="Probability of Default", ticktype = "detailed")
 

#B2 - fix amount
for(i in 1: length(xxx2))
{
  a<-1- csfebiv(Duration, xbiv, yyy, cbind(xxx[30], xxx2[i]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)
  mat.out[,i]<-a
}
persp(x=xxx2, y=yyy, z=mat.out, theta = 70, phi = 20, ylab="Duration", xlab="Age", zlab="Probability of Default", ticktype = "detailed")

#### B3 - fix age
for(i in 1: length(xxx2))
{
  a<-1- csfebiv(Duration, xbiv, yyy, cbind(xxx[i], xxx2[10]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)
  mat.out[,i]<-a
}
persp(x=xxx, y=yyy, z=mat.out, theta = 70, phi = 20, ylab="Duration", xlab="Amount", zlab="Probability of Default", ticktype = "detailed")

#### B4 - fix age
for(i in 1: length(xxx2))
{
  a<-1- csfebiv(Duration, xbiv, yyy, cbind(xxx[i], xxx2[40]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)
  mat.out[,i]<-a
}
persp(x=xxx, y=yyy, z=mat.out, theta = 70, phi = 20, ylab="Duration", xlab="Amount", zlab="Probability of Default", ticktype = "detailed")
