rm(list = ls())

german_credit <- read.csv(".../german_creditv1.0.csv") # Insert path to data file here
Duration<-german_credit[,1]
Amount<-german_credit[,2]
Censoring<- german_credit[,3] # 0 =  good credit = censored because we do not know if they are going to default
age<-german_credit[,4]

SavingsAccount<-german_credit[,5]	
EmploymentHistory	<-german_credit[,6]
InstallmentRate	<-german_credit[,7]
NoOfCreditsInBank	<-german_credit[,8]
NoOfPeopleLiable<-german_credit[,9]

yyy<-seq(min(Duration), max(Duration), length=50)
 
 
covariate <- age  
 

"IntEpanechnikov"<-function(x)
{
  ifelse(x< -2.236068, 0, ifelse(x> 2.236068, 1, .5- (2.236068*x*(x^2-15))/100))
}

"IntUnif"<-function(x)
{
  ifelse(abs(x)< 1, 0.5*(x+1), ifelse(x> 1, 1, 0))
}

"Hstar"<-function(xin, cens, xout)
{
  ordxin <- sort(xin) #order input data=Xi
  ordcens <- cens[order(xin)] #order censoring =Di
  n <- length(xin)
  nn <- length(xout)
  vKuse <- sapply(1:nn, function (i, ordxin, xout) max(c(max(which(ordxin <= xout[i]))
                                                         , min(which(xout[i] <= ordxin)))), ordxin, xout)
  vK <- vKuse - 1
  Huse<-sapply(vK, function (i,n,ordcens) ((n-1:i+1)/(n-1:i+2))^(1-ordcens[1:i]), n, ordcens)
  result<-sapply(Huse, prod)
  result 
}

"IntfHStar1"<-function(xin, cens, xout, h, kfun)
{
  n<-length(xout)
  nn<-length(xin)
  out<-vector("numeric", n)
  denominator<- Hstar(xin, 1-cens, xin)
  arg1<-cens/ denominator 
  out<-sapply(1:n, function(i, xout, xin, h,arg1, kfun) 
    sum(kfun( (xout[i]-xin)/h)*arg1), xout, xin, h, arg1, kfun)
  out /(nn *h)
}

"csfe"<-function(yin, xin, yout, xout, h, b, kfun, wfun, cens)
{
  n<- length(xin)
  arg1<-(sapply(xout, "-", xin))/h  
  arg2<-kfun(arg1)
  
  arg3<-(sapply(yout, "-", yin))/b  
  arg3.1 <- cens/Hstar(yin, 1-cens, yin) 
  arg4<-wfun(arg3)* arg3.1
  
  #cat(nrow(arg2), " ", ncol(arg2), " ", nrow(arg4), " ", ncol(arg4), "\n")
  arg2<-matrix(arg2, nrow=nrow(arg4), ncol=ncol(arg4))
  arg5<- arg2 *arg4
  colSums(arg5)/n
}

"csfebiv"<-function(yin, xin, yout, xout, h, b, kfun, wfun, cens)
{
  n<- nrow(xin)
  xin1<-xin[,1]
  xin2<-xin[,2]
  xout1<-xout[,1]
  xout2<-xout[,2]
  h1<-h[1]
  h2<-h[2]
  
  arg1.1<-(sapply(xout1, "-", xin1))/h1
  arg2.1<-kfun(arg1.1)
  
  arg1.2<-(sapply(xout2, "-", xin2))/h2
  arg2.2<-kfun(arg1.2)
  
  arg3<-(sapply(yout, "-", yin))/b
  arg3.1 <- cens/Hstar(yin, 1-cens, yin)
  arg4<-wfun(arg3)* arg3.1
  arg2.1<-matrix(arg2.1, nrow=nrow(arg4), ncol=ncol(arg4))
  arg2.2<-matrix(arg2.2, nrow=nrow(arg4), ncol=ncol(arg4))
  arg5<- arg2.1 *arg2.2 *arg4
  colSums(arg5)/n
}

"csfetriv"<-function(yin, xin, yout, xout, h, b, kfun, wfun, cens)
{
  n<- nrow(xin)
  xin1<-xin[,1]
  xin2<-xin[,2]
  xin3<-xin[,3]
  xout1<-xout[,1]
  xout2<-xout[,2]
  xout3<-xout[,3]
  h1<-h[1]
  h2<-h[2]
  h2<-h[3]
  
  arg1.1<-(sapply(xout1, "-", xin1))/h1
  arg2.1<-kfun(arg1.1)
  
  arg1.2<-(sapply(xout2, "-", xin2))/h2
  arg2.2<-kfun(arg1.2)
  
  arg1.3<-(sapply(xout3, "-", xin3))/h2
  arg2.3<-kfun(arg1.3)
  
  arg3<-(sapply(yout, "-", yin))/b
  arg3.1 <- cens/Hstar(yin, 1-cens, yin)
  arg4<-wfun(arg3)* arg3.1
  arg2.1<-matrix(arg2.1, nrow=nrow(arg4), ncol=ncol(arg4))
  arg2.2<-matrix(arg2.2, nrow=nrow(arg4), ncol=ncol(arg4))
  arg2.3<-matrix(arg2.3, nrow=nrow(arg4), ncol=ncol(arg4))
  arg5<- arg2.1 *arg2.2 *arg2.3 *arg4
  colSums(arg5)/n
}

### NUMERICAL INTEGRATION FOR ISE
"SimpsonInt"<- function(xin, h)
{   #xin = data, h=distance of the points at which function is evaluated
  n<-length(xin)
  nn<-1:n
  int0<-xin[1]
  int2n<-xin[n]
  inteven<-xin[seq(3,n-1,by=2)]
  intodd<-xin[seq(2,n-2,by=2)]
  sumodd<-sum(intodd)
  sumeven<-sum(inteven)
  res<-(h/3)*(int0+4*sumodd+2*sumeven+int2n)
  res
}


xxx<-seq(min(Amount), max(Amount), length=50)
xxx2<-seq(min(age), max(age), length=50)
xxx3<-seq(min(SavingsAccount), max(SavingsAccount), length=50)
xxx4<-seq(min(InstallmentRate), max(InstallmentRate), length=50)
xxx5<-seq(min(EmploymentHistory), max(EmploymentHistory), length=50)


xbiv<-cbind(Amount, age)
xtriv<-cbind(Amount, age, InstallmentRate)
 
mat.ise.s1<-matrix(0, 50, 50)
mat.ise.b1<-matrix(0, 50, 50)
for(i in 1:50)
{
  for(j in 1:50)
  {
xoutbiv1<-cbind( xxx[i], xxx2[j])
# MODEL S1:
cdfeS1<-  csfe(Duration, Amount, yyy, xoutbiv1[1],  bw.SJ(Amount), 3,  pnorm, IntEpanechnikov, Censoring)
 
# MODEL B1:
cdfeB1<-  csfebiv(Duration, xbiv, yyy, xoutbiv1,  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)
 
ISEs1.1<- SimpsonInt((cdfeS1-cdfeB1)^2, yyy[2]-yyy[1])
mat.ise.s1[i,j]<-ISEs1.1
###BB MODEL

xouttriv1<-cbind( xxx[i], xxx2[j], xxx4[j] )
cdfeBB1<-  csfetriv(Duration, xtriv, yyy, xouttriv1,  c(bw.SJ(covariate),bw.SJ(age), bw.SJ(InstallmentRate)), 3,  pnorm, IntEpanechnikov, Censoring)

#cdfeBB1<-  csfebiv(Duration, xtriv, yyy, xouttriv1,  c(bw.SJ(Amount),bw.nrd(age)), 3,  pnorm, IntEpanechnikov, Censoring)
ISEs1.2<- SimpsonInt((cdfeB1-cdfeBB1)^2, yyy[2]-yyy[1])
mat.ise.b1[i,j]<-ISEs1.2
  }
}
 
cat(mean(mat.ise.s1[,2:49]), " ", mean(mat.ise.b1[,2:49]), "\n")
 
mat.ise.s2<-matrix(0, 50, 50)
mat.ise.b2<-matrix(0, 50, 50)
for(i in 1:50)
{
  for(j in 1:50)
  {
    xoutbiv1<-cbind( xxx[i], xxx2[j])
    # MODEL S1:
    cdfeS2<-  csfe(Duration, age, yyy, xoutbiv1[2],  bw.SJ(age), 3,  pnorm, IntEpanechnikov, Censoring)
    
    # MODEL B1:
    cdfeB2<-  csfebiv(Duration, xbiv, yyy, xoutbiv1,  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)
    
    ISEs2<- SimpsonInt((cdfeS2-cdfeB2)^2, yyy[2]-yyy[1])
    mat.ise.s2[i,j]<-ISEs2
    ###BB MODEL
    
    xouttriv1<-cbind( xxx[i], xxx2[j], xxx4[j] )
    cdfeBB1<-  csfetriv(Duration, xtriv, yyy, xouttriv1,  c(bw.SJ(covariate),bw.SJ(age), bw.SJ(InstallmentRate)), 3,  pnorm, IntEpanechnikov, Censoring)
    
    #cdfeBB1<-  csfebiv(Duration, xtriv, yyy, xouttriv1,  c(bw.SJ(Amount),bw.nrd(age)), 3,  pnorm, IntEpanechnikov, Censoring)
    ISEb2<- SimpsonInt((cdfeB2-cdfeBB1)^2, yyy[2]-yyy[1])
    mat.ise.b2[i,j]<-ISEb2
  }
}
cat(mean(mat.ise.s1), " ", mean(mat.ise.b1), "\n") #mise s1, b1
cat(mean(mat.ise.s2), " ", mean(mat.ise.b2), "\n") #mise s2, b2

#Plot models
cdfeS1.1<-  csfe(Duration, Amount, yyy, xxx[30],  bw.SJ(Amount), 3,  pnorm, IntEpanechnikov, Censoring)
cdfeS1.2<-  csfe(Duration, Amount, yyy, xxx[40],  bw.SJ(Amount), 3,  pnorm, IntEpanechnikov, Censoring)
cdfeB1.1<-  csfebiv(Duration, xbiv, yyy, cbind(xxx[30], xxx2[18]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)
cdfeB1.2<-  csfebiv(Duration, xbiv, yyy, cbind(xxx[40], xxx2[30]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)

plot(yyy, 1-cdfeS1.1, type="l", col=4, ylim=c(0,1), xlab="Duration", ylab="Probability of Default", lwd=2)

lines(yyy, 1-cdfeB1.1, type="l", col=2,   lwd=2)

lines(yyy, 1-cdfeS1.2, type="l", col=4,   lwd=2, lty=2)
lines(yyy, 1-cdfeB1.2, type="l", col=2,   lwd=2, lty=2)

a.1<-paste0("S1(Duration | Amount=", round(xxx[30],1),"),  MISE_c = ", round(mat.ise.s1[30,1]/10, 4)  )
b.1<-paste0("B1(Duration | Amount=", round(xxx[30],1), ", Age = ", round(xxx2[18],0), "),   MISE_c = ", round(mat.ise.b1[30,18], 4) )
a.2<-paste0("S1(Duration | Amount=", round(xxx[40],1),"),  MISE_c = ", round(mat.ise.s1[40,1]/10, 4)  )
b.2<-paste0("B1(Duration | Amount=", round(xxx[40],1), ", Age = ", round(xxx2[30],0), "),   MISE_c = ", round(mat.ise.b1[40,30], 4) )

 


legend("bottomleft",  c(a.1, b.1,   a.2, b.2 ),   lty = c(1,1,   2,2 ), col=c(  4, 2, 4,2 ), lwd = c(2,2, 2,2 ), cex=1.3)



cdfeS2.1<-  csfe(Duration, age, yyy, xxx2[18],  bw.SJ(age), 3,  pnorm, IntEpanechnikov, Censoring)
cdfeS2.2<-  csfe(Duration, age, yyy, xxx2[30],  bw.SJ(age), 3,  pnorm, IntEpanechnikov, Censoring)
cdfeB2.1<-  csfebiv(Duration, xbiv, yyy, cbind(xxx[30], xxx2[18]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)
cdfeB2.2<-  csfebiv(Duration, xbiv, yyy, cbind(xxx[40], xxx2[30]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)

plot(yyy, 1-cdfeS2.1, type="l", col=4, ylim=c(0,1), xlab="Duration", ylab="Probability of Default", lwd=2)

lines(yyy, 1-cdfeB2.1, type="l", col=2,   lwd=2)

lines(yyy, 1-cdfeS2.2, type="l", col=4,   lwd=2, lty=2)
lines(yyy, 1-cdfeB2.2, type="l", col=2,   lwd=2, lty=2)

a.1<-paste0("S2(Duration | Age=", round(xxx2[18],0),"),  MISE_c = ", round(mat.ise.s2[1, 24]/10, 4)  )
b.1<-paste0("B2(Duration | Age = ", round(xxx2[18],0), ", Amount=", round(xxx[30],0), "), MISE_c = ", round(mat.ise.b1[30,18], 4) )
a.2<-paste0("S2(Duration | Age=", round(xxx2[30],0),"),  MISE_c = ", round(mat.ise.s2[1, 30]/10, 4)  )
b.2<-paste0("B2(Duration | Age = ", round(xxx2[30],0), ", Amount=", round(xxx[40],0), "),   MISE_c = ",round(mat.ise.b1[40,30], 4) )




legend("bottomleft",  c(a.1, b.1,   a.2, b.2 ),   lty = c(1,1,   2,2 ), col=c(  4, 2, 4,2 ), lwd = c(2,2, 2,2 ), cex=1.3)

xbiv<-cbind(Amount, age)
 
   
    
cdfe.alt1<-  csfe(Duration, Amount, yyy, xxx[29],  bw.SJ(Amount), 3,  pnorm, IntEpanechnikov, Censoring)
cdfe.alt2<-  csfe(Duration, Amount, yyy, xxx[39],  bw.SJ(Amount), 3,  pnorm, IntEpanechnikov, Censoring)
cdfe.altb1<-  csfebiv(Duration, xbiv, yyy, cbind(xxx[29], xxx2[29]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)
cdfe.altb2<-  csfebiv(Duration, xbiv, yyy, cbind(xxx[39], xxx2[36]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)

plot(yyy, 1-cdfe.alt1, type="l", col=4, ylim=c(0,1), xlab="Duration", ylab="Probability of Default", lwd=2)
lines(yyy, 1-cdfe.altb1, type="l", col=2,   lwd=2)
lines(yyy, 1-cdfe.alt2, type="l", col=4,   lwd=2, lty=2)
lines(yyy, 1-cdfe.altb2, type="l", col=2,   lwd=2, lty=2)

a.1<-paste0("S1(Duration | Amount=", round(xxx[29],0),"),  MISE_c = ", round(mat.ise.s1[29, 31], 4)  )
b.1<-paste0("B(Duration | Age = ", round(xxx2[29],0), ", Amount=", round(xxx[29],0), "), MISE_c = ", round(mat.ise.b1[29,31], 4) )
a.2<-paste0("S1(Duration | Amount=", round(xxx[39],0),"),  MISE_c = ", round(mat.ise.s1[39, 36], 4)  )
b.2<-paste0("B(Duration | Age = ", round(xxx2[36],0), ", Amount=", round(xxx[39],0), "),   MISE_c = ",round(mat.ise.b1[39, 36], 4) )
legend("bottomleft",  c(a.1, b.1,   a.2, b.2 ),   lty = c(1,1,   2,2 ), col=c(  4, 2, 4,2 ), lwd = c(2,2, 2,2 ), cex=1.3)



cdfe.alts21<-  csfe(Duration, age, yyy, xxx2[29],  bw.SJ(age), 3,  pnorm, IntEpanechnikov, Censoring)
cdfe.alts22<-  csfe(Duration, age, yyy, xxx2[36],  bw.SJ(age), 3,  pnorm, IntEpanechnikov, Censoring)
cdfe.altb1<-  csfebiv(Duration, xbiv, yyy, cbind(xxx[29], xxx2[29]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)
cdfe.altb2<-  csfebiv(Duration, xbiv, yyy, cbind(xxx[39], xxx2[36]),  c(bw.SJ(Amount),bw.SJ(age)), 3,  pnorm, IntEpanechnikov, Censoring)

plot(yyy, 1-cdfe.alts21, type="l", col=4, ylim=c(0,1), xlab="Duration", ylab="Probability of Default", lwd=2)
lines(yyy, 1-cdfe.altb1, type="l", col=2,   lwd=2)
lines(yyy, 1-cdfe.alts22, type="l", col=4,   lwd=2, lty=2)
lines(yyy, 1-cdfe.altb2, type="l", col=2,   lwd=2, lty=2)

a.1<-paste0("S2(Duration | Age=", round(xxx2[29],0),"),  MISE_c = ", round(mat.ise.s2[29, 31], 4)  )
b.1<-paste0("B(Duration | Age = ", round(xxx2[29],0), ", Amount=", round(xxx[29],0), "), MISE_c = ", round(mat.ise.b1[29,31], 4) )
a.2<-paste0("S2(Duration | Age=", round(xxx2[36],0),"),  MISE_c = ", round(mat.ise.s2[36, 39], 4)  )
b.2<-paste0("B(Duration | Age = ", round(xxx2[36],0), ", Amount=", round(xxx[39],0), "),   MISE_c = ",round(mat.ise.b1[39, 36], 4) )
legend("bottomleft",  c(a.1, b.1,   a.2, b.2 ),   lty = c(1,1,   2,2 ), col=c(  4, 2, 4,2 ), lwd = c(2,2, 2,2 ), cex=1.3)
